package com.biganto.visual.roompark.conductor.dialogs.tour_chooser

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.ImageView
import androidx.annotation.LayoutRes
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.dialogs.change_handler.DialogChangeHandler
import com.biganto.visual.roompark.data.repository.file.FileModule
import com.biganto.visual.roompark.di.dagger.ActivityModule
import com.biganto.visual.roompark.di.dagger.AppComponent
import com.biganto.visual.roompark.di.dagger.PerScreen
import com.biganto.visual.roompark.domain.interactor.SettingsInteractor
import com.biganto.visual.roompark.domain.model.TourModel
import com.biganto.visual.roompark.domain.model.bytesToSize
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonRecyclerAdapter
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonViewHolder
import com.biganto.visual.roompark.util.extensions.setGone
import com.biganto.visual.roompark.util.view_utils.snackbar.ISnackBarProvider
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.DownloadState
import com.bluelinelabs.conductor.Controller
import com.bluelinelabs.conductor.RouterTransaction
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.google.android.material.textview.MaterialTextView
import com.jakewharton.rxbinding3.view.clicks
import dagger.Binds
import dagger.BindsInstance
import dagger.Component
import dagger.Module
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 09.04.2019.
 */

private const val TOTAL_SIZE_DOWNLOAD_KEY = "TOURS_TO_DOWNLOAD_OVERALL_SIZE"

class StartToursDownloadingDialogController : Controller {

    constructor(args: Bundle) : super(args)

    constructor(size:Long) : super(Bundle().apply { putLong(TOTAL_SIZE_DOWNLOAD_KEY,size) } )


    private val detachDisposable = CompositeDisposable()

    override fun onDetach(view: View) {
        detachDisposable.clear()
        super.onDetach(view)
    }


    @Inject
    lateinit var downloader: SettingsInteractor

    @Inject
    lateinit var file: FileModule

    lateinit var snackbar: ISnackBarProvider

    @Inject
    lateinit var rpActivity: RoomParkMainActivity

    override fun onContextAvailable(context: Context) {
        super.onContextAvailable(context)

        DaggerStartToursDownloadingDialogScreenComponent.factory()
            .create(RoomParkApplication.component,activity as RoomParkMainActivity)
            .inject(this)
    }


    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup): View {
        val view = inflater.inflate(getLayoutId(), container, false)

        snackbar = ActivityModule.provideSnackBar(rpActivity)

        args.getLong(TOTAL_SIZE_DOWNLOAD_KEY).let {
            view.findViewById<MaterialTextView>(R.id.all_tours_size_textView)
                .text = resources?.getString(R.string.download_all_tours_size_title,it.bytesToSize())
        }

        file.freeSpace.bytesToSize().let {
            view.findViewById<MaterialTextView>(R.id.disk_size_textView)
                .text = resources?.getString(R.string.download_all_tours_disk_size_title,it)
        }

        view.findViewById<Button>(R.id.alert_dismiss_button)
            .setOnClickListener { handleBack() }

        view.findViewById<Button>(R.id.close_current_button)
            .setOnClickListener { handleBack() }

        detachDisposable.add(
            view.findViewById<Button>(R.id.alert_ok_button)
                .clicks()
                .debounce(200,TimeUnit.MILLISECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .flatMapCompletable { downloader.startToursDownloading() }
                .subscribe ({
                    snackbar.showSnackBar(R.string.download_all_tours_start_snackbar_message)
                    handleBack()
                },{
                    Timber.e(it)
                    snackbar.showSnackBar(R.string.download_all_tours_errors_snackbar_message)
                    handleBack()
                }
                )
        )

//        view.findViewById<View>(R.id.close_current_button).setOnClickListener { handleBack() }
        view.setOnClickListener { handleBack() }

        return view
    }


    @LayoutRes
    fun getLayoutId() = R.layout.tours_download_dialog_screen

    override fun handleBack(): Boolean {
        return router.popCurrentController()
    }
}

@PerScreen
@Component(
    modules = [StartToursDownloadingDialogScreenModule::class],
    dependencies = [AppComponent::class])
interface StartToursDownloadingDialogComponent {

    @Component.Factory
    interface Factory{
        fun create(
            appComponent: AppComponent
            ,@BindsInstance activity: RoomParkMainActivity
        ): DownloaderScreenComponent
    }

    fun inject(controller: StartToursDownloadingDialogController)
}

@Module
abstract class StartToursDownloadingDialogScreenModule{

    @PerScreen
    @Binds
    abstract fun provideContext(activity: RoomParkMainActivity): Context
}
