package com.biganto.visual.roompark.domain.interactor

import android.content.Intent
import android.os.Build
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.BaseRoomParkActivity
import com.biganto.visual.roompark.data.service.download.DOWNLOAD_MANAGER_ADD_IDS_TO_LOAD_COMMAND
import com.biganto.visual.roompark.data.service.download.DOWNLOAD_MANAGER_COMMAND_KEY
import com.biganto.visual.roompark.data.service.download.DownloadManagerService
import com.biganto.visual.roompark.domain.model.CachedDataModel
import com.biganto.visual.roompark.domain.model.PushSwitchModel
import com.biganto.visual.roompark.domain.model.SettingsModel
import com.biganto.visual.roompark.domain.model.SubscriptionModel
import com.biganto.visual.roompark.domain.use_case.*
import io.reactivex.Completable
import io.reactivex.Observable
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 09.10.2019.
 */

class SettingsInteractor @Inject constructor(
    private val auth: AuthUseCase,
    private val settingsUseCase: SettingsUseCase,
    private val activity: BaseRoomParkActivity,
    private val subUc: SubscriptionUseCase,
    private val toursUc: TourPreviewsUseCase,
    private val planTypes: PlanTypesUseCase
){

    fun downloadTourPlans(cancellationToken:DownloadUseCase.CancellationToken)
        = planTypes.downloadAllPlanTypes(cancellationToken)

    fun fetchToursSizes() = toursUc.fetchToursSizes()

    fun fetchPlanTypesSizes() = planTypes.fetchNotDownloadedPlansSizes()

    private fun startDownloadService() {
        val i = Intent(activity, DownloadManagerService::class.java)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            activity.startForegroundService(i)
        } else {
            activity.startService(i)
        }
    }

    private fun startDownloadService(ids:List<String>){
        Timber.d(" gonna startService ++")
        try {

            val i = Intent(activity, DownloadManagerService::class.java)
            i.putExtra(DOWNLOAD_MANAGER_COMMAND_KEY, DOWNLOAD_MANAGER_ADD_IDS_TO_LOAD_COMMAND)
            i.putStringArrayListExtra(TOUR_IDS_TO_DOWNLOAD_KEY, ArrayList(ids))

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                activity.startForegroundService(i)
            } else {
                activity.startService(i)
            }
        }catch (e:Exception){
            Timber.d(" gonna startService errror")
            Timber.e(e)
        }
    }

    fun startToursDownloading(): Completable =
        toursUc.downloadAllDeelsAndEstates()
            .doOnNext { Timber.d(" gonna startService") }
            .doOnNext { this.startDownloadService() }
            .delay(100,TimeUnit.MILLISECONDS)
        .doOnNext { tours ->
            startDownloadService(tours.map { tour -> tour.id })
        }.ignoreElements()


    fun getSubscriptions() =
        subUc.getCurrentUserSubscriptions()

    fun switchSubscription(model: SubscriptionModel, newState: Boolean): Completable =
        when(newState){
            true -> subUc.subscribeTopic(model.id,model.topic)
            false -> subUc.unSubscribeTopic(model.id,model.topic)
        }


    private val plans : Observable<CachedDataModel>
            get() = settingsUseCase.planTypesSize.map {
                CachedDataModel(activity.resources.getString(R.string.plans_cache), it, 1)
    }

    private val tours
        get() = settingsUseCase.toursSize.map {
            CachedDataModel(activity.resources.getString(R.string.tours_cache),it,2)
        }

    private val feeds
        get() = settingsUseCase.feedsSize.map {
            CachedDataModel(activity.resources.getString(R.string.feeds_cache),it,3)
        }


    private val albums
        get() = settingsUseCase.albumsSize.map {
            CachedDataModel(activity.resources.getString(R.string.albums_cache),it,4)
        }

    private val overall
        get() = settingsUseCase.overallSize.map {
            CachedDataModel(activity.resources.getString(R.string.overall_cache),it,0)
        }

    fun deleteCacheFiles() = settingsUseCase.clearAllCache()

    fun getCacheInfo(): Observable<MutableList<CachedDataModel>> =
        Observable.concatArray(plans, tours, feeds, albums, overall).toList().toObservable()

    fun fetchSettings(): Observable<SettingsModel> = Observable.just(sampleSettings)

    fun signOut() = auth.signOut().subscribeOn(Schedulers.io())

    companion object{

        val sampleSettings = SettingsModel(
            arrayListOf(

                PushSwitchModel("Новости",1,"zsldfj",true),
                PushSwitchModel("Блог",2,"zsldfj",false),
                PushSwitchModel("Ход Строительства Дом №1",3,"zsldfj",true),
                PushSwitchModel("Сделка кв №451",4,"zsldfj",true),
                PushSwitchModel("Сделка кв №452",5,"zsldfj",true)
            ),
            arrayListOf(
                CachedDataModel("Скачать карточки моих\n" +
                        "квартир из избранного\n" +
                        "и сделок",8415456588L,900),
                CachedDataModel("Скачать виртуальные туры \n" +
                        "моих квартир из избранного\n" +
                        "и сделок ",656853321588L,999)

            ),
            arrayListOf(
                CachedDataModel("Всего скачано",2834264238L,1),
                CachedDataModel("Карточки квартир",6434268L,2),
                CachedDataModel("Виртуальные туры",1782234268L,3),
                CachedDataModel("Новости и заметки",4323438L,4),
                CachedDataModel("Фотографии",164444L,5)
            )
        )
    }
}
